<?php
namespace Khatfield\SoapClient;

final class Events
{
    const REQUEST    = 'khatfield.soap_client.request';
    const RESPONSE   = 'khatfield.soap_client.response';
    const FAULT      = 'khatfield.soap_client.fault';
}

