<?php

namespace Khatfield\SoapClient\Result;

class DescribeGlobalSObjectResult
{
    public $activatable;
    public $createable;
    public $custom;
    public $customSetting;
    public $deletable;
    public $deprecatedAndHidden;
    public $feedEnabled;
    public $keyPrefix;
    public $label;
    public $labelPlural;
    public $layoutable;
    public $mergeable;
    public $name;
    public $queryable;
    public $replicateable;
    public $retrieveable;
    public $searchable;
    public $triggerable;
    public $undeletable;
    public $updateable;
}