<?php

namespace Khatfield\SoapClient\Result;

class LeadConvertResult
{
    public $accountId;
    public $contactId;
    public $leadId;
    public $opportunityId;
    public $success;
    public $errors;
}

